/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const express = require('express');

const PnadNovoApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const config = require(`${libs}/config`); 

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

PnadNovoApp.use(cache('15 day'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'id',
    table: 'pnad_novo',
    tableField: 'id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: ['nome', 'id'],
    resultField: ['state_name', 'state_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_uf',
        table: 'pnad_novo'
    },
    join: {
        primary: 'id',
        foreign: 'cod_uf',
        foreignTable: 'pnad_novo'
    }
}).addValue({
    name: 'state_not',
    table: 'estado',
    tableField: ['nome', 'id'],
    resultField: ['state_name', 'state_id'],
    where: {
        relation: '<>',
        type: 'integer',
        field: 'cod_uf',
        table: 'pnad_novo'
    },
    join: {
        primary: 'id',
        foreign: 'cod_uf',
        foreignTable: 'pnad_novo'
    }
}).addValue({
    name: 'illiteracy',
    table: 'pnad_novo',
    tableField: 'analfabetismo',
    resultField: 'illiteracy',
    where: {
        relation: '=',
        type: 'integer',
        field: 'analfabetismo'
    }
}).addValue({
    name: 'years_of_study',
    table: 'pnad_novo',
    tableField: 'anos_de_estudo',
    resultField: 'years_of_study',
    where: {
        relation: '=',
        type: 'integer',
        field: 'years_of_study'
    }
}).addValue({
    name: 'min_year',
    table: 'pnad_novo',
    tableField: 'ano_ref',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_ref'
    }
}).addValue({
    name: 'max_year',
    table: 'pnad_novo',
    tableField: '',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_ref'
    }
}).addField({
    name: 'search',
    field: false,
    where: true
}).addValueToField({
    name: 'name',
    table: 'pnad_novo',
    tableField: 'nome',
    where: {
        relation: 'LIKE',
        type: 'string',
        field: 'nome'
    }
}, 'search').addValue({
    name: 'mesoregion',
    table: 'pnad_novo',
    tableField: 'mesorregiao_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'mesorregiao_id'
    }
}).addValue({
    name: 'microregion',
    table: 'pnad_novo',
    tableField: 'microrregiao_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'microrregiao_id'
    }
}).addValueToField({
    name: 'region',
    table: 'estado',
    tableField: 'regiao_id',
    resultField: 'region_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'regiao_id',
        table: 'estado'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'municipio'
    }
}, 'filter');

PnadNovoApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.from('pnad_novo')
    .field('round(sum(pnad_novo.peso_domicilio_pessoas_com_cal), 0)', 'name')
    .field('pnad_novo.ano_ref', 'year')
    .group('pnad_novo.ano_ref')
    console.log(req.sql.toString())
    next();
}, query, response('pnad_novo'));


module.exports = PnadNovoApp;
